"""A modification workflow for migrating an ImportedSite to a Site subscription."""

from orchestrator.targets import Target
from orchestrator.types import State, UUIDstr
from orchestrator.workflow import StepList, done, init, step, workflow
from orchestrator.workflows.steps import resync, store_process_subscription, unsync
from orchestrator.workflows.utils import wrap_modify_initial_input_form

from gso.products import ProductName
from gso.products.product_types.site import ImportedSite, Site
from gso.services.subscriptions import get_product_id_by_name


@step("Create new site subscription")
def import_site_subscription(subscription_id: UUIDstr) -> State:
    """Take an ImportedSite subscription, and turn it into a Site subscription."""
    old_site = ImportedSite.from_subscription(subscription_id)
    new_subscription_id = get_product_id_by_name(ProductName.SITE)
    new_subscription = Site.from_other_product(old_site, new_subscription_id)  # type: ignore[arg-type]

    return {"subscription": new_subscription}


@workflow("Import Site", target=Target.MODIFY, initial_input_form=wrap_modify_initial_input_form(None))
def import_site() -> StepList:
    """Modify an ImportedSite subscription into a Site subscription to complete the import."""
    return init >> store_process_subscription(Target.MODIFY) >> unsync >> import_site_subscription >> resync >> done
